@@ -54,7 +54,33 @@ struct _GMountTracker
   DBusConnection *connection;
 };
 
-G_DEFINE_TYPE (GMountTracker, g_mount_tracker, G_TYPE_OBJECT);
+/* We use this hack to avoid problems when this code
+   is shared in both the daemon and the client */
+static GType _g_mount_tracker_get_type (void) G_GNUC_CONST;
+
+#define g_mount_tracker_get_type _g_mount_tracker_get_type
+G_DEFINE_TYPE (GMountTracker, g_mount_tracker, G_TYPE_OBJECT)
+#undef g_mount_tracker_get_type
+
+GType
+g_mount_tracker_get_type (void)
+{
+  static volatile gsize type_volatile = 0;
+  
+  if (g_once_init_enter (&type_volatile))
+    {
+      GType type;
+      
+      type = g_type_from_name ("GMountTracker");
+      if (type == 0)
+	type = _g_mount_tracker_get_type ();
+      
+      g_once_init_leave (&type_volatile, type);
+    }
+  
+  return type_volatile;
+}
+
 
 static DBusHandlerResult g_mount_tracker_filter_func  (DBusConnection        *conn,
 						       DBusMessage           *message,
@@ -89,6 +115,7 @@ g_mount_info_dup (GMountInfo *info)
   copy->ref_count = 1;
   copy->display_name = g_strdup (info->display_name);
   copy->stable_name = g_strdup (info->stable_name);
+  copy->x_content_types = g_strdup (info->x_content_types);
   copy->icon = g_strdup (info->icon);
   copy->dbus_id = g_strdup (info->dbus_id);
   copy->object_path = g_strdup (info->object_path);
@@ -114,6 +141,7 @@ g_mount_info_unref (GMountInfo *info)
     {
       g_free (info->display_name);
       g_free (info->stable_name);
+      g_free (info->x_content_types);
       g_free (info->icon);
       g_free (info->dbus_id);
       g_free (info->object_path);
@@ -158,6 +186,7 @@ g_mount_info_from_dbus (DBusMessageIter *iter)
   dbus_bool_t user_visible;
   char *display_name;
   char *stable_name;
+  char *x_content_types;
   char *icon;
   char *prefered_filename_encoding;
   char *dbus_id;
@@ -174,6 +203,7 @@ g_mount_info_from_dbus (DBusMessageIter *iter)
 				      DBUS_TYPE_OBJECT_PATH, &obj_path,
 				      DBUS_TYPE_STRING, &display_name,
 				      DBUS_TYPE_STRING, &stable_name,
+                                      DBUS_TYPE_STRING, &x_content_types,
 				      DBUS_TYPE_STRING, &icon,
 				      DBUS_TYPE_STRING, &prefered_filename_encoding,
 				      DBUS_TYPE_BOOLEAN, &user_visible,
@@ -191,6 +221,7 @@ g_mount_info_from_dbus (DBusMessageIter *iter)
   info->ref_count = 1;
   info->display_name = g_strdup (display_name);
   info->stable_name = g_strdup (stable_name);
+  info->x_content_types = g_strdup (x_content_types);
   info->icon = g_strdup (icon);
   info->dbus_id = g_strdup (dbus_id);
   info->object_path = g_strdup (obj_path);
@@ -210,7 +241,8 @@ g_mount_tracker_finalize (GObject *object)
 
   tracker = G_MOUNT_TRACKER (object);
 
-  g_mutex_free (tracker->lock);
+  if (tracker->lock)
+    g_mutex_free (tracker->lock);
   
   g_list_foreach (tracker->mounts,
 		  (GFunc)g_mount_info_unref, NULL);
@@ -333,15 +365,21 @@ static void
 g_mount_tracker_add_mount (GMountTracker *tracker,
 			   GMountInfo *info)
 {
-  g_mutex_lock (tracker->lock);
+  if (tracker->lock)
+    g_mutex_lock (tracker->lock);
   
   /* Don't add multiple times */
   if (g_mount_tracker_find (tracker, info))
-    return;
+    {
+      if (tracker->lock)
+	g_mutex_unlock (tracker->lock);
+      return;
+    }
 
   tracker->mounts = g_list_prepend (tracker->mounts, g_mount_info_ref (info));
 
-  g_mutex_unlock (tracker->lock);
+  if (tracker->lock)
+    g_mutex_unlock (tracker->lock);
   
   g_signal_emit (tracker, signals[MOUNTED], 0, info);
 }
@@ -353,20 +391,25 @@ g_mount_tracker_remove_mount (GMountTracker *tracker,
   GList *l;
   GMountInfo *old_info;
 
-  g_mutex_lock (tracker->lock);
-  
+  if (tracker->lock)
+    g_mutex_lock (tracker->lock);
   
   l = g_mount_tracker_find (tracker, info);
   
   /* Don't remove multiple times */
   if (l == NULL)
-    return;
+    {
+      if (tracker->lock)
+	g_mutex_unlock (tracker->lock);
+      return;
+    }
 
   old_info = l->data;
   
   tracker->mounts = g_list_delete_link (tracker->mounts, l);
   
-  g_mutex_unlock (tracker->lock);
+  if (tracker->lock)
+    g_mutex_unlock (tracker->lock);
 
   g_signal_emit (tracker, signals[UNMOUNTED], 0, old_info);
   g_mount_info_unref (old_info);
